# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/daml_lf_dev/daml_lf_0.proto

from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="com/daml/daml_lf_dev/daml_lf_0.proto",
    package="daml_lf_0",
    syntax="proto3",
    serialized_options=b"\n\024com.daml.daml_lf_dev\252\002\034Com.Daml.Daml_Lf_Dev.DamlLf0",
    serialized_pb=b'\n$com/daml/daml_lf_dev/daml_lf_0.proto\x12\tdaml_lf_0"\t\n\x07PackageB5\n\x14\x63om.daml.daml_lf_dev\xaa\x02\x1c\x43om.Daml.Daml_Lf_Dev.DamlLf0b\x06proto3',
)


_PACKAGE = _descriptor.Descriptor(
    name="Package",
    full_name="daml_lf_0.Package",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=51,
    serialized_end=60,
)

DESCRIPTOR.message_types_by_name["Package"] = _PACKAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Package = _reflection.GeneratedProtocolMessageType(
    "Package",
    (_message.Message,),
    {
        "DESCRIPTOR": _PACKAGE,
        "__module__": "com.daml.daml_lf_dev.daml_lf_0_pb2"
        # @@protoc_insertion_point(class_scope:daml_lf_0.Package)
    },
)
_sym_db.RegisterMessage(Package)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
