# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Main module that exposes utility functions that can be called directly from the command line.
"""

from .cli import main

main()
